// AD7798-Treiber   Dr. Weisser 26.3.2019  22.10.2019
//***************************************************************************
//  File........: ad7798.c
//  Target(s)...: ATmega2560
//  Compiler....: AVR-GCC
//  Description.: ADC routines for AD7714 analog digital converter
//  Hardware:     Platine mega2560R3-Shield mit AD7798 
//                Referenz 2.5V -> unipolar 0-2.5V, bipolar -2.5 ... 2.5V
//                Zugriff per SPI 8MHz
//***************************************************************************
/*  26.3.2019 Anpassung Treiber von AD7714 auf AD7798
              Register und Pins angepasst auf AD7798
              statt adc_write() nun spi_send_rcv(). CS_L und CS_H dazu beachten
              nun wird nicht vor jedem Byte CS low und nach jedem Byte CS high 
              Calibration mit 16 bit statt 24 bit 
              Compilerfehler weg. 
              dafuer viele Linkerfehler:
              undefined reference to spi_send_rcv(). Warum? 
    27.3.2019 im Makefile fehlt spi.c -> daher die Fehler 
              uebersetzt nun fehlerfrei 
              kein weiterer Test momentan moeglich da kein chip/Leiterplatte da 
    30.6.2019 Pins geaendert fuer Atmega32U4, compiliert fehlerfrei 
   20.10.2019 Debug-Ausgaben outs(USART,PSTR) -> outs_p(USART,PSTR) 
              Test erscheint fehlerhaft, beide Kanaele zeigen Maximum obwohl 
              8.2k Widerstand am Eingang. 
   21.10.2019 AD7798_DIN_H(); -> DIN-Pin auf high - vgl. Datenblatt S.20
              adc_reset(); -> wartet 500us vor Registerzugriff hinzu 
              Versuch continous mode abzuschalten in adc_init(); 
*/

#include <stdint.h>       // Integer-Datentypen 8-64bit
#include <avr/io.h>       // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include "main.h"         // CPU-Frequenz, sbi, cbi
#include "delay.h"        // Wartezeiten 2us, ms
#include "ad7798.h"       // ADC-Pins, ADC-Funktionen + Prototypen
#include "spi.h"          // SPI-Funktionen + Prototypen
#include "out.h"          // Ausgabe-Funktionen Zeichen, Zahlen
#include "usart.h"        // USART-Funktionen
#include <avr/pgmspace.h> // PSTR


#define DEBUG      0      // Debug-Ausgaben einschalten: 1 oder ausschalten: 0



void adc_init(void)   // AD7798 initialisieren
                      // Pins init, SPI-Mode 3, SPI-Takt 8MHz, ADC-Reset, Calib
                      // SPI nicht aktiv danach
{
   unsigned char data_h, data_l;
   
   // ADC-Pins init 
   AD7798_CS_OUT();       // CS-Pin auf Ausgang
   AD7798_CS_H();         // CS high setzen

   AD7798_SCK_OUT();      // SCK-Pin auf Ausgang
   AD7798_SCK_H();        // SCK hi setzen fuer SPI-Mode 3

   AD7798_DIN_OUT();      // DIN-Pin auf Ausgang
   //AD7798_DIN_H();      // DIN-Pin auf high - vgl. Datenblatt S.20
   AD7798_DIN_L();        // DIN-Pin besser auf low - vgl. Datenblatt S.23

   AD7798_DOUT_IN();      // DOUT-Pin auf Eingang
   AD7798_DOUT_H();       // DOUT-Pin auf high=pullup aktiv

   // SPI-Mode 3 sicherstellen
   spi_mode3();  // vgl. Datenblatt S.7 SCK normalerweise 1, Datenuebernahme steigende SCK-Flanke 

   // SPI-Takt 8 MHz
   spi_speed8(); 
   
   // SPI aktiv schalten
   spi_on(); 

   // ADC-Interface sicherheitshalber ruecksetzen 
   adc_reset(); // reset: schickt 4 bytes 0xff, wartet 500us vor Registerzugriff lt. Datenblatt S.23
   
   // CH1 konfigurieren, Selbstkalibrierung

   // vor Schreiben CS low setzen, am Ende dann high
   AD7798_CS_L();         // CS low setzen

   // erst continous mode abschalten? 
   // abschalten durch Read Data register mit CR bit =0?
   //spi_send_rcv(AD7798_DATA_REG | AD7798_READ); 
   //data_h=spi_send_rcv(0x00); // lese erst h-byte (dabei 0x00 senden)
   //data_l=spi_send_rcv(0x00); // lese dann l-byte (dabei 0x00 senden)
   
   // write to communication register:
   // next write to config register
   spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE); 

   // write to config register: 16bit -> 2 bytes schreiben
   // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
   spi_send_rcv(AD7798_CH1_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
   spi_send_rcv(AD7798_CH1_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);

   // write to communication register:
   // next write to mode register
   spi_send_rcv(AD7798_MODE_REG | AD7798_WRITE);

   // write to mode register: 16bit -> 2 bytes schreiben
   // Wandelrate mit FS0-FS2 setzen, Mode mit MD0-MD2 setzen: 
   // Eingaenge werden intern kurzgeschlossen bei Calib? oder muss das sein? Kanalwahl? 
   spi_send_rcv(AD7798_CLK_17_HI | AD7798_ZS_CALIB_HI);
   spi_send_rcv(AD7798_CLK_17_LO | AD7798_ZS_CALIB_LO);

   // soll 9x 1/output rate dauern lt. Datenblatt S.25 AD7714
   // - 3x 1/output rate: fuer zero scale calibration
   // - 3x 1/output rate: fuer full scale calibration
   // - 3x 1/output rate: AD-Wandlung, setzen der Koeffizienten
   // - bei output rate CLK_50/s=20ms -> 9x 20ms= 180ms
   // kompletter Ablauf sollte also 180ms dauern, 200ms sollten reichen

   // Abfrage mit 2 while-Schleifen lt. Internet-Beispiel
   //while(! AD7714_DRDY); // warten solange DRDY 0= ready ist - ggf. mit delay warten
   //while(AD7714_DRDY);   // warte solange DRDY 1= busy ist: dauert ca. 180ms
                           // wozu dieser seltsame Konstrukt?

   // Abfrage mit while-Schleife lt. Datenblatt
   // kommt DRDY-Signal wirklich an wenn CS high ist?
   //while(AD7714_DRDY);   // warte solange DRDY 1= busy ist: dauert ca. 180ms

   // Delay statt Abfrage DRDY-Abfrage vgl. Wong
   // 180ms sollten reichen bei CLK_50
   
   // Idle-Mode: damit Schreiben Zero/Offset + Full-Scale-Register moeglich
   // write to communication register:
   // next write to mode register
   spi_send_rcv(AD7798_MODE_REG | AD7798_WRITE);

   // write to mode register: 16bit -> 2 bytes schreiben
   // Wandelrate mit FS0-FS2 setzen, Mode mit MD0-MD2 setzen: 
   spi_send_rcv(AD7798_CLK_17_HI | AD7798_IDLE_HI);
   spi_send_rcv(AD7798_CLK_17_LO | AD7798_IDLE_LO);

   delay_ms(200); // sicherheitshalber etwas laenger warten auf Ende selfcalibration




   // CH2 konfigurieren, Selbstkalibrierung

   // write to communication register:
   // next write to config register
   spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);

   // write to config register: 16bit -> 2 bytes schreiben
   // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
   spi_send_rcv(AD7798_CH2_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
   spi_send_rcv(AD7798_CH2_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);

   // write to communication register:
   // next write to mode register
   spi_send_rcv(AD7798_MODE_REG | AD7798_WRITE);

   // write to mode register: 16bit -> 2 bytes schreiben
   // Wandelrate mit FS0-FS2 setzen, Mode mit MD0-MD2 setzen: 
   // Eingaenge werden intern kurzgeschlossen bei Calib? oder muss das sein? Kanalwahl? 
   spi_send_rcv(AD7798_CLK_17_HI | AD7798_ZS_CALIB_HI);
   spi_send_rcv(AD7798_CLK_17_LO | AD7798_ZS_CALIB_LO);

   // soll 9x 1/output rate dauern lt. Datenblatt S.25 AD7714
   // - 3x 1/output rate: fuer zero scale calibration
   // - 3x 1/output rate: fuer full scale calibration
   // - 3x 1/output rate: AD-Wandlung, setzen der Koeffizienten
   // - bei output rate CLK_50/s=20ms -> 9x 20ms= 180ms
   // kompletter Ablauf sollte also 180ms dauern, 200ms sollten reichen

   // Abfrage mit 2 while-Schleifen lt. Internet-Beispiel
   //while(! AD7714_DRDY); // warten solange DRDY 0= ready ist - ggf. mit delay warten
   //while(AD7714_DRDY);   // warte solange DRDY 1= busy ist: dauert ca. 180ms
                           // wozu dieser seltsame Konstrukt?

   // Abfrage mit while-Schleife lt. Datenblatt
   // kommt DRDY-Signal wirklich an wenn CS high ist?
   //while(AD7714_DRDY);   // warte solange DRDY 1= busy ist: dauert ca. 180ms

   // Delay statt Abfrage DRDY-Abfrage vgl. Wong
   // 180ms sollten reichen bei CLK_50
   delay_ms(200); // sicherheitshalber etwas laenger warten auf Ende selfcalibration

   // CS am Ende wieder high
   AD7798_CS_H();         // CS high setzen

   // SPI wieder deaktiv schalten
   spi_off(); 

   // CS sollte nun high sein.

}



void adc_reset(void)  // ADC-Interface reset
                      // SPI-Mode 3 noetig !
                      // SPI-Speed muss passen
                      // SPI muss aktiv sein
{
   AD7798_CS_L();      // CS low

   // 4 bytes per SPI zum ADC senden 
   spi_send_rcv(0xFF);  // sendet 4 bytes ff (32 clock-Zyklen) lt. Datenblatt S.13,19,23
   spi_send_rcv(0xFF);
   spi_send_rcv(0xFF);
   spi_send_rcv(0xFF);

   AD7798_CS_H();      // CS high
   
   delay_us(500); // 500us warten nach Softwarereset vor Registerzugriff lt. Datenblatt S.19,23
}


/*
void adc_write(unsigned char byte)  // ein byte an ADC senden
                                    // SPI-Mode 3 noetig !
                                    // SPI-Speed muss passen
                                    // SPI muss aktiv sein
                                    // CS muss low sein
                                    // CS nicht vergessen wieder high zu setzen
{

   spi_send_rcv(byte);  // sendet byte

} */



unsigned int adc_read(unsigned char channel) // ADC Messung starten, warten, auslesen
                                             // SPI-Mode 3 noetig !
                                             // SPI-Speed muss passen
                                             // SPI muss aktiv sein
                                             // CS wird hier low, high gesetzt 
{
   unsigned char data_h, data_l;
   unsigned int data=0;
   
   // CS low machen fuer Zugriff auf Register und Daten
   AD7798_CS_L();      // CS low 

   if(channel==CHANNEL_2){
      // Kanal 2
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH2_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH2_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
   }
   else if(channel==CHANNEL_3){ // Kanal 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH3_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH3_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
   }
   else {
      // Kanal 1, falls nicht 2 oder 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH1_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH1_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
   }

   // Single conversion-Mode: Wandlung dauert 2/fadc lt. Datenblatt S.15
   // write to communication register: next write to mode register
   spi_send_rcv(AD7798_MODE_REG | AD7798_WRITE);
   
   // write to mode register: 16bit -> 2 bytes schreiben
   // Wandelrate mit FS0-FS2 setzen, Mode mit MD0-MD2 setzen: 
   spi_send_rcv(AD7798_CLK_17_HI | AD7798_SINGLE_HI);
   spi_send_rcv(AD7798_CLK_17_LO | AD7798_SINGLE_LO);

   // warten bis Wandlung fertig ist 
   //while(AD7714_DRDY);    // warten solange DRDY high ist
   // 60ms warten sollte reichen bei 50/s. 
   //delay_ms(65);
   delay_ms(200); 

   //data_h=spi_send_rcv(0x00); // lese erst h-byte (dabei 0x00 senden)
   data_h=spi_send_rcv(0x80); // lese erst h-byte (dabei 0x80 senden)

#if DEBUG
   outs_p(USART,PSTR("data_h: ")); // Meldung ausgeben
   out_u08(USART,data_h,0,0);
   outs_p(USART,PSTR("\n")); // Meldung ausgeben
#endif // DEBUG

   //data_l=spi_send_rcv(0x00); // lese dann l-byte (dabei 0x00 senden)
   data_h=spi_send_rcv(0x80); // lese erst h-byte (dabei 0x80 senden)

#if DEBUG
   outs_p(USART,PSTR("data_l: ")); // Meldung ausgeben
   out_u08(USART,data_l,0,0);
   outs_p(USART,PSTR("\n")); // Meldung ausgeben
#endif // DEBUG

   AD7798_CS_H();         // CS high

   // h-byte 8 bit nach links schieben, dann l-byte dazu
   data= ((((unsigned int)data_h) << 8) | (unsigned int)data_l);

   return (data); // 2 bytes als unsigned int
}



unsigned int adc_read_zero_scale(unsigned char channel) // Calib Offset/ZS lesen 
                                             // SPI-Mode 3 noetig !
                                             // SPI-Speed muss passen
                                             // SPI muss aktiv sein
{
   unsigned char data_h, data_l;
   unsigned int data=0;

   // CS low machen fuer Zugriff auf Register und Daten
   AD7798_CS_L();      // CS low 

   if(channel==CHANNEL_2){
      // Kanal 2
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH2_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH2_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: read from offset register Kanal 2
      spi_send_rcv(AD7798_ZERO_REG | AD7798_READ);
   }
   else if(channel==CHANNEL_3){
      // Kanal 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH3_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH3_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: read from offset register Kanal 3
      spi_send_rcv(AD7798_ZERO_REG | AD7798_READ);
   }
   else {
      // Kanal 1, falls nicht 2 oder 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH1_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH1_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: read from offset register Kanal 3
      spi_send_rcv(AD7798_ZERO_REG | AD7798_READ);
   }
   
   data_h=spi_send_rcv(0x00); // h-byte holen
   data_l=spi_send_rcv(0x00); // l-byte holen

   AD7798_CS_H();    // CS high

   // bytes zusammenbauen zu int-Wert
   // h-byte dazu 8bit links schieben
   // l-byte nicht schieben
   data= ( (((unsigned int)data_h) << 8) |
            ((unsigned int)data_l) );

   return data;
}



void adc_write_zero_scale(unsigned int zs, unsigned char channel) // Calib Offset/ZS schreiben
                                             // SPI-Mode 3 noetig !
                                             // SPI-Speed muss passen
                                             // SPI muss aktiv sein
                                             // muss Idle-Mode oder PowerDown-Mode sein
{
   unsigned char data_h, data_l;
   unsigned int data;

   // CS low machen fuer Zugriff auf Register und Daten
   AD7798_CS_L();      // CS low 

   // 2 Teile a 8 bit =16bit fuer zero scale

   data=zs; // Daten zwischenspeichern

   // data in 2 bytes zerlegen um diese zu senden
   data_l= (unsigned char) data;        // untere 8bit nehmen
   data=zs;
   data_h= (unsigned char) (data >>8);  // obere 8bit 8bit nach unten schieben


   if(channel==CHANNEL_2){
      // Kanal 2
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH2_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH2_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: write to zero/offset register Kanal 2
      spi_send_rcv(AD7798_ZERO_REG | AD7798_WRITE);
   }
   else if(channel==CHANNEL_3){
      // Kanal 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH3_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH3_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: write to zero/offset register Kanal 3
      spi_send_rcv(AD7798_ZERO_REG | AD7798_WRITE);
   }
   else {
      // Kanal 1, falls nicht 2 oder 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH1_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH1_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: write to zero/offset register Kanal 1
      spi_send_rcv(AD7798_ZERO_REG | AD7798_WRITE);
   }

   // 2 bytes per SPI zum ADC senden 
   spi_send_rcv(data_h); // erst MSB
   spi_send_rcv(data_l);

   AD7798_CS_H();      // CS high
}



unsigned int adc_read_full_scale(unsigned char channel) // Calib FS lesen 
                                             // SPI-Mode 3 noetig !
                                             // SPI-Speed muss passen
                                             // SPI muss aktiv sein
{
   unsigned char data_h, data_l;
   unsigned long data=0;

   // CS low machen fuer Zugriff auf Register und Daten
   AD7798_CS_L();      // CS low 

   if(channel==CHANNEL_2){
      // Kanal 2
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH2_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH2_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: read from gain register Kanal 2
      spi_send_rcv(AD7798_FULL_REG | AD7798_READ);
   }
   else if(channel==CHANNEL_3){
      // Kanal 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH3_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH3_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: read from gain register Kanal 3
      spi_send_rcv(AD7798_FULL_REG | AD7798_READ);
   }
   else {
      // Kanal 1, falls nicht 2 oder 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH1_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH1_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: read from gain register Kanal 1
      spi_send_rcv(AD7798_FULL_REG | AD7798_READ);
   }

   data_h=spi_send_rcv(0x00);  // byte holen, dazu 0x00 senden
   data_l=spi_send_rcv(0x00);  // byte holen, dazu 0x00 senden

   AD7798_CS_H();   // CS high

   // bytes zusammenbauen zu int-Wert
   // h-byte dazu 8bit links schieben
   // l-byte nicht mehr schieben
   data= ( (((unsigned int)data_h) << 8) |
            ((unsigned long)data_l) );

   return data;
}



void adc_write_full_scale(unsigned int fs, unsigned char channel) // Calib FS schreiben
                                             // SPI-Mode 3 noetig !
                                             // SPI-Speed muss passen
                                             // SPI muss aktiv sein
                                             // muss Idle-Mode oder PowerDown-Mode sein
{
   unsigned char data_h, data_l;
   unsigned long data;

   // CS low machen fuer Zugriff auf Register und Daten
   AD7798_CS_L();      // CS low 

   // 16bit int in 2 Teile a 8 bit =16bit fuer zero scale

   data=fs; // Daten zwischenspeichern

   // data in 2 bytes zerlegen um diese zu senden
   data_l= (unsigned char) data;        // untere 8bit nehmen
   data=fs;
   data_h= (unsigned char) (data >>8);  // obere 8bit 8bit nach unten schieben

   if(channel==CHANNEL_2){
      // Kanal 2
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH2_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH2_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: write to FS register Kanal 2
      spi_send_rcv(AD7798_FULL_REG | AD7798_WRITE);
   }
   else if(channel==CHANNEL_3){
      // Kanal 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH3_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH3_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: write to FS register Kanal 3
      spi_send_rcv(AD7798_FULL_REG | AD7798_WRITE);
   }
   else {
      // Kanal 1, falls nicht 2 oder 3
      // write to communication register: next write to config register
      spi_send_rcv(AD7798_CONF_REG | AD7798_WRITE);
      // write to config register: 16bit -> 2 bytes schreiben
      // Kanalwahl CH0-CH2, kein BUFFER, Gain mit G0-G2, unipolar: 
      spi_send_rcv(AD7798_CH1_HI | AD7798_BUF_OFF_HI | AD7798_GAIN_2_HI | AD7798_UNIPOLAR_HI);
      spi_send_rcv(AD7798_CH1_LO | AD7798_BUF_OFF_LO | AD7798_GAIN_2_LO | AD7798_UNIPOLAR_LO);
      
      // write to communication register: write to FS register Kanal 1
      spi_send_rcv(AD7798_FULL_REG | AD7798_WRITE);
   }

   // 3 bytes per SPI zum ADC senden 
   spi_send_rcv(data_h);  // erst MSB
   spi_send_rcv(data_l);

   AD7798_CS_H();   // CS high
}



void zero_full_scale_adj(unsigned char channel)  // Zero/Full Scale nachstellen 
                                                 // SPI-Mode 3 noetig !
                                                 // SPI-Speed muss passen
                                                 // SPI muss aktiv sein
{
   unsigned int zs,fs;

   zs=adc_read_zero_scale(channel);
   zs-=5000;   // input bias voltage compensation
   adc_write_zero_scale(zs,channel);

   fs=adc_read_full_scale(channel);
   fs-=5000;   // input bias voltage compensation
   adc_write_full_scale(fs,channel);
}



//void spi_send_byte(unsigned char byte)  // byte zum ADC senden
//nutze spi_send_rcv(unsigned char byte) stattdessen -> spi.c
                    // CS muss vorher low sein -> dies sicherstellen
                    // DRDY muss low sein -> dies vorher sicherstellen
                    // SCK muss low sein -> dies vorher sicherstellen
                    // SPI-Mode 1 noetig !
                    // SPI-Speed muss passen
                    // SPI muss aktiv sein

//unsigned char spi_receive_byte(void)  // byte vom ADC empfangen
//nutze unsigned char spi_send_rcv(0x00) stattdessen -> spi.c
                    // DRDY muss dazu low sein -> dies vorher sicherstellen
                    // CS muss ebenfalls low sein -> dies vorher sicherstellen
                    // SCK muss vorher low sein -> dies vorher sicherstellen
                    // SPI-Mode 1 noetig !
                    // SPI-Speed muss passen
                    // SPI muss aktiv sein



void adc_test(void)  // ADC-Test. vorher sollte adc_init() gelaufen sein
{
   unsigned int data; 
   unsigned char i; 
   
   // korrekten SPI-Mode sicherstellen
   spi_mode3(); // Mode 3
   
   // Meldung fuer ADC-Test ausgeben
   outs_p(USART,PSTR("\nADC-Test ausfuehren")); 
   
   // SPI aktiv schalten
   spi_on();    // SPI aktiv schalten
   
   for (i=1; i<21; i++){  // 20 Messungen von 1-20
      outs_p(USART,PSTR("\nMessung ")); // Meldung ausgeben
      out_u08(USART,i,2,0);
      outs_p(USART,PSTR(":\n")); // Meldung ausgeben

      // ADC-Wert von Kanal 1 einlesen
      data =adc_read(CHANNEL_1); 

      // ADC-Kanal 1 ausgeben
      outs_p(USART,PSTR("ADC-CH1: ")); // Meldung ausgeben
      out_u16(USART,data,0,0);
      outs_p(USART,PSTR("\n")); // Meldung ausgeben

      // ADC-Wert von Kanal 2 einlesen
      data =adc_read(CHANNEL_2); 

      // ADC-Kanal 2 ausgeben
      outs_p(USART,PSTR("ADC-CH2: ")); // Meldung ausgeben
      out_u16(USART,data,0,0);
      outs_p(USART,PSTR("\n")); // Meldung ausgeben

      delay_ms(100);  // damit weniger Werte
   }
   
   // SPI deaktiv schalten 
   spi_off();     // SPI deaktiv schalten 
   
   // Schlussmeldung ausgeben
   outs_p(USART,PSTR("\nADC-Test beendet\n")); // Meldung ausgeben
   
} 

