//ad7798.h

#ifndef AD7798_H_
#define AD7798_H_


// Pins am Mikrocontroller
// CS-Pin AD7798   PG5 bei mega2560, PB6 bei ATmega32U4 
#define AD7798_CS_PORT      PORTB
#define AD7798_CS_DDR       DDRB
#define AD7798_CS_PIN       PINB
#define AD7798_CS_BIT       6                  // Pin 6 am Port
#define AD7798_CS_OUT()     sbi(DDRB,6)        // Ausgabe
//#define AD7798_CS_H()     PORTB |= (1<<6)    // CS hi
#define AD7798_CS_H()       sbi(PORTB,6)       // CS hi
//#define AD7798_CS_L()     PORTB &= ~(1<<6)   // CS low
#define AD7798_CS_L()       cbi(PORTB,6)       // CS low
#define AD7798_CS_VAL       bit_is_set(PINB,6) // CS level

// SCK-Pin AD7798
#define AD7798_SCK_PORT     PORTB
#define AD7798_SCK_DDR      DDRB
#define AD7798_SCK_PIN      PINB
#define AD7798_SCK_BIT      1                  // Pin 1 am Port
#define AD7798_SCK_OUT()    sbi(DDRB,1)        // Ausgabe
//#define AD7798_SCK_H()    PORTB |= (1<<1)    // CLK hi
#define AD7798_SCK_H()      sbi(PORTB,1)       // CLK hi
//#define AD7798_SCK_L()    PORTB &= ~(1<<1)   // CLK low
#define AD7798_SCK_L()      cbi(PORTB,1)       // CLK low
#define AD7798_SCK_VAL      bit_is_set(PINB,1) // CLK level

// DIN-Pin AD7798
#define AD7798_DIN_PORT     PORTB
#define AD7798_DIN_DDR      DDRB
#define AD7798_DIN_PIN      PINB
#define AD7798_DIN_BIT      2                  // Pin 2 am Port
#define AD7798_DIN_OUT()    sbi(DDRB,2)        // Ausgabe
//#define AD7798_DIN_H()    PORTB |= (1<<2)    // DIN hi
#define AD7798_DIN_H()      sbi(PORTB,2)       // DIN hi
//#define AD7798_DIN_L()    PORTB &= ~(1<<2)   // DIN low setzen
#define AD7798_DIN_L()      cbi(PORTB,2)       // DIN low
#define AD7798_DIN_VAL      bit_is_set(PINB,2) // DIN level

// DOUT-Pin AD7798
#define AD7798_DOUT_PORT    PORTB
#define AD7798_DOUT_DDR     DDRB
#define AD7798_DOUT_PIN     PINB
#define AD7798_DOUT_BIT     3                  // Pin 3 am Port
#define AD7798_DOUT_IN()    cbi(DDRB,3)        // Eingabe
#define AD7798_DOUT_H()     sbi(PORTB,3)       // DOUT hi-pullup
//#define AD7798_DOUT_VAL   (PINB & (1<<3))    // xxxx1xxx PB3: test for DO = high
#define AD7798_DOUT_VAL     bit_is_set(PINB,3) // DOUT level
/*
// Communication-Register: Schreiben 8 bits
#define CREAD 2  // 0000 0100 Continuous Read
#define RS0   3  // 0000 1000 Registerwahl
#define RS1   4  // 0001 0000 Registerwahl
#define RS2   5  // 0010 0000 Registerwahl
#define READ  6  // 0100 0000 Read/Write Register (1 = READ)
#define WEN   7  // 1000 0000 Write Disable (Write Enable falls 0)
*/

// Communication-Register CREAD: continuous read (1. Schreibzugriff)
                               //       CREAD
#define AD7798_CREAD      0x04 // 0000 0100 -> continuous read data register (when data reg selected)

// Communication-Register RS0-RS2: Registerauswahl (1. Schreibzugriff)
                               //   RS2
                               //    RS1
                               //      RS0
#define AD7798_COMM_REG   0x00 // 0000 0000 -> write communication register, read status register 8bit
#define AD7798_MODE_REG   0x08 // 0000 1000 -> mode register 16bit
#define AD7798_CONF_REG   0x10 // 0001 0000 -> configuration register 16bit
#define AD7798_DATA_REG   0x18 // 0001 1000 -> data register 16bit
#define AD7798_ID_REG     0x20 // 0010 0000 -> ID register 8bit
#define AD7798_IO_REG     0x28 // 0010 1000 -> IO register 8bit (fuer CH3-Pins Input/Output)
#define AD7798_ZERO_REG   0x30 // 0011 0000 -> nullpunkt register 16bit
#define AD7798_FULL_REG   0x38 // 0011 1000 -> full scale register 16bit

// Communication-Register RW: Schreib-/Leseumschaltung (1. Schreibzugriff)
                               //  RW
#define AD7798_WRITE      0x00 // 0000 0000  -> Register schreiben
#define AD7798_READ       0x40 // 0100 0000  -> Register/Daten lesen

/*
// Communication-Register: Lesen 8 bits (=Statusregister) 
#define CH0   0  // 0000 0001 Channel converted
#define CH1   1  // 0000 0010 Channel converted
#define CH2   2  // 0000 0100 Channel converted
#define NOREF 5  // 0010 0000 keine Referenz da
#define ERR   6  // 0100 0000 Fehler overrange/underrange etc
#define RDY   7  // 1000 0000 Data ready
*/

/*
// Mode-Register: 16 bits 
#define FS0   0  // 0000 0000 0000 0001 filter select
#define FS1   1  // 0000 0000 0000 0010 filter select. nach Reset
#define FS2   2  // 0000 0000 0000 0100 filter select
#define FS3   3  // 0000 0000 0000 1000 filter select. nach Reset
#define PSW   12 // 0001 0000 0000 0000 power switch control
#define MD0   13 // 0010 0000 0000 0000 mode select: conversion/calibration 
#define MD1   14 // 0100 0000 0000 0000 mode select: conversion/calibration 
#define MD2   15 // 1000 0000 0000 0000 mode select: conversion/calibration 
*/

// Mode-Register FS0-FS3: Filtergrenzfrequenz, Messrate
// Filterfreq. lt. Datenblatt S.15, Table 14 
                          //                FS3
                          //                 FS2
                          //                  FS1
                          //                   FS0
                          // 0000 0000 0000 0001 -> 470/s 4ms Einschwingzeit
#define AD7798_CLK_470_HI         0x00
#define AD7798_CLK_470_LO                   0x01
                          // 0000 0000 0000 0010 -> 242/s 8ms Einschwingzeit
#define AD7798_CLK_242_HI         0x00
#define AD7798_CLK_242_LO                   0x02
                          // 0000 0000 0000 0011 -> 123/s 16ms Einschwingzeit
#define AD7798_CLK_123_HI         0x00
#define AD7798_CLK_123_LO                   0x03
                          // 0000 0000 0000 0100 -> 62/s 32ms Einschwingzeit
#define AD7798_CLK_62_HI          0x00
#define AD7798_CLK_62_LO                    0x04
                          // 0000 0000 0000 0101 -> 50/s 40ms Einschwingzeit
#define AD7798_CLK_50_HI          0x00
#define AD7798_CLK_50_LO                    0x05
                          // 0000 0000 0000 0111 -> 33.2/s 60ms Einschwingzeit
#define AD7798_CLK_33_HI          0x00
#define AD7798_CLK_33_LO                    0x07
                          // 0000 0000 0000 1000 -> 19.6/s 101ms Einschwingzeit: 90dB 60Hz-Unterdrueckung
#define AD7798_CLK_20_HI          0x00
#define AD7798_CLK_20_LO                    0x08
                          // 0000 0000 0000 1001 -> 16.7/s 120ms Einschwingzeit: 80dB 50Hz-Unterdrueckung
#define AD7798_CLK_17_HI          0x00
#define AD7798_CLK_17_LO                    0x09
                          // 0000 0000 0000 1100 -> 10/s 200ms Einschwingzeit: 69dB 50/60Hz
#define AD7798_CLK_10_HI          0x00
#define AD7798_CLK_10_LO                    0x0c
                          // 0000 0000 0000 1111 -> 4.17/s 480ms Einschwingzeit: 74dB 50/60Hz
#define AD7798_CLK_4_HI           0x00
#define AD7798_CLK_4_LO                     0x0f

// Mode-Register MD0-MD2: Betriebsart - operating mode lt. Datenblatt S.15, Table 13 
                          // MD2
                          //  MD1
                          //   MD0
                          // 0000 0000 0000 0000 -> continuous conversion
#define AD7798_NORMAL_HI          0x00
#define AD7798_NORMAL_LO                    0x00
                          // 0010 0000 0000 0000 -> single conversion
#define AD7798_SINGLE_HI          0x20
#define AD7798_SINGLE_LO                    0x00
                          // 0100 0000 0000 0000 -> idle mode
#define AD7798_IDLE_HI            0x40
#define AD7798_IDLE_LO                      0x00
                          // 0110 0000 0000 0000 -> power down mode
#define AD7798_PWRDN_HI           0x60
#define AD7798_PWRDN_LO                     0x00
                          // 1000 0000 0000 0000 -> internal zero scale calibration
#define AD7798_ZS_CALIB_HI        0x80
#define AD7798_ZS_CALIB_LO                  0x00
                          // 1010 0000 0000 0000 -> internal full scale calibration
#define AD7798_FS_CALIB_HI        0xa0
#define AD7798_FS_CALIB_LO                  0x00
                          // 1100 0000 0000 0000 -> system zero scale calibration
#define AD7798_ZS_SYS_CALIB_HI    0xc0
#define AD7798_ZS_SYS_CALIB_LO                  0x00
                          // 1110 0000 0000 0000 -> system full scale calibration
#define AD7798_FS_SYS_CALIB_HI    0xe0
#define AD7798_FS_SYS_CALIB_LO              0x00

/*
// Configuration-Register: 16 bits 
#define CH0     0  // 0000 0000 0000 0001 channel select
#define CH1     1  // 0000 0000 0000 0010 channel select
#define CH2     2  // 0000 0000 0000 0100 channel select
#define BUF     4  // 0000 0000 0001 0000 buffer enable
#define REF_DET 5  // 0000 0000 0010 0000 reference detect enable
#define G0      8  // 0000 0001 0000 0000 gain select
#define G1      9  // 0000 0010 0000 0000 gain select
#define G2      10 // 0000 0100 0000 0000 gain select
#define UB      12 // 0001 0000 0000 0000 unipolar/bipolar select
#define BO      13 // 0010 0000 0000 0000 burnout current enable
*/

// Configuration-Register CH0-CH2: Kanalauswahl lt. Datenblatt S.16, Table 15 
                          //                 CH2
                          //                  CH1
                          //                   CH0
                          // 0000 0000 0000 0000 -> Diff-Messung AIN1+/AIN1-: Calib Reg Pair 0
#define AD7798_CH1_HI             0x00
#define AD7798_CH1_LO                       0x00
                          // 0000 0000 0000 0001 -> Diff-Messung AIN2+/AIN2-: Calib Reg Pair 1
#define AD7798_CH2_HI             0x00
#define AD7798_CH2_LO                       0x01
                          // 0000 0000 0000 0010 -> Diff-Messung AIN3+/AIN3-: Calib Reg Pair 2
#define AD7798_CH3_HI             0x00
#define AD7798_CH3_LO                       0x02
                          // 0000 0000 0000 0011 -> Diff-Messung AIN1-/AIN1-: Calib Reg Pair 0
#define AD7798_CH1_TST_HI         0x00
#define AD7798_CH1_TST_LO                   0x03
                          // 0000 0000 0000 0111 -> AVDD-Monitor: Gain=1/6, intern. Ref 1.17V
#define AD7798_CH_AVDD_HI         0x00
#define AD7798_CH_AVDD_LO                   0x07

// Configuration-Register BUF: Buffer lt. Datenblatt S.16, Table 15 
                          //              BUF
                          // 0000 0000 0001 0000 -> Buffer on
#define AD7798_BUF_ON_HI          0x00
#define AD7798_BUF_ON_LO                    0x10
                          // 0000 0000 0000 0000 -> Buffer off
#define AD7798_BUF_OFF_HI         0x00
#define AD7798_BUF_OFF_LO                   0x00

// Configuration-Register G0-G2: gain setting lt. Datenblatt S.16, Table 15 
                          //       G2
                          //        G1
                          //         G0
                          // 0000 0000 0000 0000 -> Gain=1 (ohne IN-AMP) 2.5V-Bereich
#define AD7798_GAIN_1_HI          0x00
#define AD7798_GAIN_1_LO                    0x00
                          // 0000 0001 0000 0000 -> Gain=2 (ohne IN-AMP) 1.25V-Bereich
#define AD7798_GAIN_2_HI          0x01
#define AD7798_GAIN_2_LO                    0x00
                          // 0000 0010 0000 0000 -> Gain=4 (mit IN-AMP) 625mV-Bereich (Verlust 1V oben)
#define AD7798_GAIN_4_HI          0x02
#define AD7798_GAIN_4_LO                    0x00
                          // 0000 0011 0000 0000 -> Gain=8 (mit IN-AMP) 312.5mV-Bereich (Verlust 1V oben)
#define AD7798_GAIN_8_HI          0x03
#define AD7798_GAIN_8_LO                    0x00
                          // 0000 0100 0000 0000 -> Gain=16 (mit IN-AMP) 156.2mV-Bereich (Verlust 1V oben)
#define AD7798_GAIN_16_HI         0x04
#define AD7798_GAIN_16_LO                   0x00
                          // 0000 0101 0000 0000 -> Gain=32 (mit IN-AMP) 78.125mV-Bereich (Verlust 1V oben)
#define AD7798_GAIN_32_HI         0x05
#define AD7798_GAIN_32_LO                   0x00
                          // 0000 0110 0000 0000 -> Gain=64 (mit IN-AMP) 39.06mV-Bereich (Verlust 1V oben)
#define AD7798_GAIN_64_HI         0x05
#define AD7798_GAIN_64_LO                   0x00
                          // 0000 0111 0000 0000 -> Gain=128 (mit IN-AMP) 19.53mV-Bereich (Verlust 1V oben)
#define AD7798_GAIN_128_HI        0x07
#define AD7798_GAIN_128_LO                  0x00

// Configuration-Register UB: unipolar/bipolar setting lt. Datenblatt S.16, Table 15 
                          //    UB
                          // 0000 0000 0000 0000 -> bipolare Codierung +-2.5V-Bereich
#define AD7798_BIPOLAR_HI         0x00
#define AD7798_BIPOLAR_LO                   0x00
                          // 0001 0000 0000 0000 -> unipolare Codierung 0-2.5V-Bereich
#define AD7798_UNIPOLAR_HI        0x10
#define AD7798_UNIPOLAR_LO                  0x00

// Configuration-Register BO: burnout currents: geht nur wenn Buffer/IN-AMP aktiv ist 
                          //   BO
                          // 0010 0000 0000 0000 -> burnout currents on
#define AD7798_BURN_HI            0x20
#define AD7798_BURN_LO                      0x00



// Kanaele
#define CHANNEL_1   1  // Kanal 1
#define CHANNEL_2   2  // Kanal 2
#define CHANNEL_3   3  // Kanal 3


//extern unsigned int status_register;

void adc_init(void);                  // Pins init, SPI-Mode 1, SPI 8 MHz, ADC reset, calib 
void adc_reset(void);                 // ADC-Interface ruecksetzen 
//void adc_write(unsigned char byte);   // CS low, byte an ADC schicken, CS high -> spi_send_rcv()
unsigned int adc_read(unsigned char channel); // Wandlung starten, warten, Wert einlesen
unsigned int adc_read_zero_scale(unsigned char);        // Offset/ZS ADC lesen
void adc_write_zero_scale(unsigned int,unsigned char);  // Offset/ZS ADC schreiben
unsigned int adc_read_full_scale(unsigned char);        // FS ADC lesen
void adc_write_full_scale(unsigned int,unsigned char);  // FS ADC schreiben
void zero_full_scale_adj(unsigned char);                // Korrektur Offset ZS,FS
//unsigned char spi_receive_byte(void);                 // ein byte lesen -> spi.c
//void spi_send_byte(unsigned char byte);               // ein byte schreiben -> spi.c
void adc_test(void);                                    // ADC-Test

#endif
