// AD7798-AD-Wandler-Projekt   Dr. Weisser 26.3.2019  1.11.2019
//***************************************************************************
//  File........: main.c
//  Target(s)...: ATmega2560 -> ATmega32u4
//  Compiler....: AVR-GCC
//  Description.: AD7798 auf ATmega32u4
//                übersetzt mit Makefile: make clean, make all
//                per USB auf ATmega32u4: make program
//***************************************************************************
/*  26.3.2019 Uebernahme von AD7714
              Pins fuer AD7798 definiert in ad7798.h und main.c
              Messraten und Filter festgelegt
              init_adc() -> adc_init() 
              i2c_init() auskommentiert 
   22.10.2019 outs(USART,PSTR) -> outs_p(USART,PSTR) 
              Ausgabe CS-Signal am ADC klappt 
              keine Ausgabe SPI -> spi_init(); unten eingebaut -> Signale kommen nun 
              spi_send_rcv(0x08); korrekt zu sehen - Datenblatt S.20
              spi_send_rcv(0x20); korrekt zu sehen
              spi_send_rcv(0x0a); korrekt zu sehen - auch alle 3 hintereinander 
              SPI-Takt 8MHz zu sehen - auch korrekt 
              Programmierung genau nach Beispiel S.20 
              Lesen Daten mit spi_send_rcv(0x00) liefert immer ffff. Warum? 
              -> Versuch mit Lesen Daten spi_send_rcv(0xff) liefert mal
              65535 und dann mehrmals 32768. 
              der Eingang jedoch sollte auf Masse liegen. 
              der Ausgang des ADC scheint auf high zu haengen. Warum? 
   29.10.2019 neues AD-Wandler-Chip eingebaut -> selbes Problem ! 
              Softwareproblem? alle beiden Kanaele liefern 65535 statt 0 
              USART-Ausgaben gehen nicht. Laptop neu booten ! 
              USART-Ausgaben kommen nun 
              ADC liefert noch immer nur 65535. Pin DOUT scheint dauernd high !
              DOUT haengt auch am SRAM. zieht SRAM den Pin hoch? 
              der erste Wert ist 0. DOUT ist also nicht immer high ! siehe Oszi 
              danach geht DOUT auf high, bleibt high 
              CS SRAM sollte immer high sein - ist auch so laut Oszi 
              es muss also ein Softwareproblem sein 
              delay_ms(200) statt delay_ms(70) liefert Werte 0 
              delay_ms(100) reicht nicht 
              delay_ms(120) reicht nicht 
              delay_ms(130) reicht nicht 
              delay_ms(140) reicht nicht 
              delay_ms(145) reicht nicht 
              delay_ms(150) reicht nicht 
              komisch. nun liefert auch delay_ms(200) keinen Wert 0 mehr 
              delay_ms(70) liefert nun Bereichsmitte 32768 - wieso? 
              spi_send_rcv(0xff) sendete bisher immer 1111 1111 
              spi_send_rcv(0x00) sendet nun immer 0000 0000 
              spi_send_rcv(0x80) sendet nun immer 1000 0000 
              kein grosser Unterschied zu sehen 
              Bereichsmitte 32768 sollte ok sein weil bipolar nach Reset 
              Abfrage Status klappt nicht 
   30.10.2019 Versuch mit adc_read(unsigned char channel) + 200ms warten 
              es kommen Werte 0, 0, 0, 65535, 0, 65535, 0, 0, 1023
              komisch !
              data_h=spi_send_rcv(0x00) -> data_h=spi_send_rcv(0x80); 
              es kommen Werte 0, 0, 0, 65280, 0, 0 
              keine Aenderung der Werte wenn Finger am Eingang 
              Unsinnswerte - nach Abziehen USB-Stecker wieder andere 
              keine Bereichsmitte mehr wie gestern - was ist da los? 
              fehlt noch ein Software-Reset? 
              Hardwareproblem mit Wackelkontakt an einem Pin? 
              fehlt etwas Loetzinn noch? 
              nun kommt wieder data=0 bei 1. Wert, dann immer 65535 
              auch das war schon mal da. 
   31.10.2019 Chip nachgeloetet. 1. Wert nun 63, dann 65535 
              keine Verbesserung zu sehen. 
              spi_send_rcv(0x08); etc. geht offenbar nicht brauchbar 
    1.11.2019 adc_read(CHANNEL_1); klappt auch nicht 
              mal kommt 0, dann wieder 65280 = 1111 1111 0000 0000 - wieso? 
              irgendwie verhaspelt sich das Teil - woran liegt das? 

*/

#include <stdint.h>        // Integer-Datentypen 8-64bit
#include <avr/io.h>        // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include <avr/interrupt.h> // Interrupts
#include "main.h"          // CPU-Frequenz, sbi, cbi
#include "timer.h"         // Timer fuer Performancemessungen
#include "usart.h"         // USART-Funktionen
#include "out.h"           // Ausgabe-Funktionen Zahlen, USART
#include "ad7798.h"        // ADC-Funktionen
#include "spi.h"           // SPI-Funktionen
#include "delay.h"         // Delay-Funktionen us, ms
#include <avr/pgmspace.h>  // PSTR

#define DEBUG 1 // Debug-Ausgaben einschalten: 1 oder ausschalten: 0

/* fuer Timing-Tests */
#if DEBUG
#endif // DEBUG 

// Global variables
volatile unsigned int Timer;  // 16bit-Timer fuer Wartezeiten (10ms increment)
                        // 0 -> 0ms
                        // 1 -> 10ms
                        // 10 -> 100ms
                        // 100 -> 1s
                        // 1000 -> 10s
                        // 10000 -> 100s

// Private functions
//

/*****************************************************************************
*   Function name : main
*   Returns :       none
*   Parameters :    none
*   Purpose :       contains main loop of the program
*****************************************************************************/

int main(void)
{
   unsigned int temp16;  // Timer
   unsigned char data_h, data_l; 
   unsigned char i; 
   unsigned int data; 

   // Initalisierung CPU, Pins in/out, ADC, USART
   initialization();  // init CPU, Pins in/out, USART

   //sei();  // Interrupts freigeben

/*   while (1){
      outs_p(USART,PSTR("\nstarte Init ADC")); // Meldung ausgeben
      outs_p(USART,PSTR("\nstarte Init ADC")); // Meldung ausgeben
   } */
   
   outs_p(USART,PSTR("\nstarte Init ADC")); // Meldung ausgeben
   
   AD7798_CS_OUT();       // CS-Pin auf Ausgang
   AD7798_CS_H();         // CS high setzen

   AD7798_SCK_OUT();      // SCK-Pin auf Ausgang
   AD7798_SCK_H();        // SCK hi setzen fuer SPI-Mode 3

   AD7798_DIN_OUT();      // DIN-Pin auf Ausgang
   //AD7798_DIN_H();      // DIN-Pin auf high - vgl. Datenblatt S.20
   AD7798_DIN_L();        // DIN-Pin besser auf low - vgl. Datenblatt S.23

   AD7798_DOUT_IN();      // DOUT-Pin auf Eingang
   AD7798_DOUT_H();       // DOUT-Pin auf high=pullup aktiv

   
   // SPI-Mode 3 sicherstellen
   spi_mode3();  // vgl. Datenblatt S.7 SCK normalerweise 1, Datenuebernahme steigende SCK-Flanke 

   // SPI-Takt 8 MHz
   spi_speed8(); 
   
   // SPI aktiv schalten
   spi_on(); 
/*   
   while(1){
     AD7798_CS_L();         // CS low setzen 
     AD7798_CS_H();         // CS high setzen 
   } */
   

   while(1){
      temp16 =adc_read(CHANNEL_1); 

      outs_p(USART,PSTR("\nADC-CH1: ")); // Meldung ausgeben
      out_u16(USART,temp16,0,0);
   } 
   
   
   while(1){
     // AD-Wandlung nach Datenblatt S.20 
     AD7798_CS_L();         // CS low setzen 
     AD7798_CS_L();         // CS low setzen 
     
     // Schreiben Mode.Reg vorbereiten 
     spi_send_rcv(0x08); // Schreiben Comm.Reg, ModeReg, kein CR 
     
     // Schreiben Mode.Reg 
     spi_send_rcv(0x20); // SingleConv, Switch off, 
     spi_send_rcv(0x0a); // 16.7Hz 120ms Einschwingen 
     
     // warten bis Wandlung fertig 
     // dazu entweder feste Zeit abwarten 
     //delay_ms(10);       // bei 16.7Hz sollten 70ms reichen 
     delay_ms(200);    // bei 16.7Hz sollten 70ms reichen 
     
/*
     // oder Statusregister auswerten 
     for (i=0; i<20; i++){ 
        spi_send_rcv(0x40); // Schreiben Comm.Reg, Read StatusReg, kein CR 0100 0000 
        data_l=spi_send_rcv(0x80);  // Read Statusregister 
        outs_p(USART,PSTR("\nStatus: ")); // Meldung ausgeben
        out_u08(USART,data_l,0,0); 
        if (data_l>=0x80){  // Statusbit gesetzt? 
           break; 
        } 
        delay_ms(10); 
     } */
     
     // 16bit Daten sollten nun da sein - bereit zum Abholen 
     // Daten holen 
     data_h=spi_send_rcv(0x80); // Lesen Daten - erst high 
     data_l=spi_send_rcv(0x80); // Lesen Daten - nun low 
     data= ((((unsigned int)data_h) << 8) | (unsigned int)data_l);
     
     // Daten per USART ausgeben 
     outs_p(USART,PSTR("\ndata_h: ")); // Meldung ausgeben
     out_u08(USART,data_h,0,0);
     
     outs_p(USART,PSTR("\ndata_l: ")); // Meldung ausgeben
     out_u08(USART,data_l,0,0);
     
     outs_p(USART,PSTR("\ndata: ")); // Meldung ausgeben
     out_u16(USART,data,0,0);
     
     //delay_us(100); 
     //delay_us(1);
     AD7798_CS_H();         // CS high setzen
     AD7798_CS_H();         // CS high setzen
   } 
   
   
   adc_init();        // ADC starten

   outs_p(USART,PSTR("\nInit ADC fertig")); // Meldung ausgeben

   while(1){
      // ADC-Wert von Kanal 1 einlesen
      temp16 =adc_read(CHANNEL_1); 

      outs_p(USART,PSTR("\nADC-CH1: ")); // Meldung ausgeben
      out_u16(USART,temp16,0,0);
      outs_p(USART,PSTR("\n")); // Meldung ausgeben

      // ADC-Wert von Kanal 2 einlesen
      temp16 =adc_read(CHANNEL_2); 

      outs_p(USART,PSTR("\nADC-CH2: ")); // Meldung ausgeben
      out_u16(USART,temp16,0,0);
      outs_p(USART,PSTR("\n")); // Meldung ausgeben

      delay_ms(100);  // nicht noetig, nur damit weniger Werte
   }
   return 1;
}



/*****************************************************************************
*   Function name : initialization
*   Returns :       none
*   Parameters :    none
*   Purpose :       initialize different modules
*****************************************************************************/
void initialization(void)
{

   // Prescaler einstellen fuer 16 MHz bei 16 MHz-Quarz 
   // set Clock Prescaler Change Enable
   CLKPR = (1<<CLKPCE);  // Bitmuster von 1 wird um CLKPCE=7 Stellen nach links geschoben 
                         // und dann ins Register CLKPR geschrieben. 1000000 also 
   // Prescaler = 1 -> 16 MHz bei 16 MHz Quarz
   CLKPR = (0<<CLKPS1) | (0<<CLKPS0);  // laeuft mit 16 MHz
                         
   //CPU_PRESCALE(0);    // siehe #define in main.h: (CLKPR = 0x80, CLKPR = (n))
                         // erst 1000 0000, dann 0000 0000 fuer ClockDivFactor 1 -> 16 MHz
                         // vgl. Datenblatt ATmega16U4/32U4 2016 S.39 

   // Disable Analog Comparator (power save)
   ACSR = (1<<ACD);      // bit 7 = ACD im Analog Comp Status Register auf 1 setzen schaltet aus
                         // Bitmuster von 1 wird ACD=7 Stellen nach links geschoben. 1000000 also.
                         // spart Strom weil Analog Comp aus.

   // Disable Digital input on analog inputs (power save)
   // DIDR0 = (1<<ADC5D) | (1<<ADC4D) | (1<<ADC3D) | (1<<ADC2D) | (1<<ADC1D) | (1<<ADC0D); //
                         // Digital Input Disable Register 0
                         // schaltet digitale Eingangspuffer der als ADC benutzten Eingänge ab
                         // um Strom zu sparen -> kein digital Input dort

   // Disable Digital input on Ain0 und Ain1 (power save)
   // DIDR1 = (0<<AIN1D) | (0<<AIN0D); // Dig. Inputs werden gebraucht, also nicht Buffer abschalten
                         // Digital Input Disable Register 1

   // DDRB Pins teilweise auf Eingabe
   DDRB = (0<<DDB7) | (1<<DDB6) | (1<<DDB5) | (0<<DDB4) | (0<<DDB3) | (1<<DDB2) | (1<<DDB1) | (1<<DDB0);
            // PB7 = D11                            - in
            // PB6 = D10      CS ADC                - out
            // PB5 = D9/A8    CS SRAM               - out
            // PB4 = D8                             - in
            // PB3 = D14 MISO ADC/SRAM              - in
            // PB2 = D16 MOSI ADC/DAC/SRAM          - out
            // PB1 = D15 SCK  ADC/DAC/SRAM          - out
            // PB0 = D17 SS (gibt SPI frei) RX LED  - out

   // PORTB pullup enable fuer Eingänge
   PORTB = (1<<PB7) | (1<<PB6) | (1<<PB5) | (1<<PB4) | (1<<PB3) | (0<<PB2) | (0<<PB1) | (0<<PB0);
            // PortB Daten Register: Enable pullup
            // damit sind bei PortB-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRC Pins teilweise auf Eingabe: kein PC0,PC1,PC2,PC3,PC4,PC5
   DDRC = (0<<DDC7) | (1<<DDC6);
            // PC7 = D13                            - in
            // PC6 = D5       LDAC DAC              - out
            // PC5 = nicht da                       - in
            // PC4 = nicht da                       - in
            // PC3 = nicht da                       - in
            // PC2 = nicht da                       - in
            // PC1 = nicht da                       - in
            // PC0 = nicht da                       - in

   // PORTC pullup enable fuer Eingänge: kein PC0,PC1,PC2,PC3,PC4,PC5
   PORTC = (1<<PC7) | (1<<PC6);
            // PortC Daten Register: Enable pullup
            // damit sind bei PortC-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRD Pins teilweise auf Eingabe
   DDRD = (0<<DDD7) | (0<<DDD6) | (1<<DDD5) | (0<<DDD4) | (1<<DDD3) | (0<<DDD2) | (0<<DDD1) | (0<<DDD0);
            // PD7 = D6/A7                          - in
            // PD6 = D12/A10                        - in
            // PD5 = TX LED                         - out
            // PD4 = D4/A6                          - in
            // PD3 = D1 TXD1                        - out
            // PD2 = D0 RXD1                        - in
            // PD1 = D2 SDA   FRAM/RTC/LCD          - in
            // PD0 = D3 SCL   FRAM/RTC/LCD          - in

   // PORTD pullup enable fuer Eingänge
   PORTD = (1<<PD7) | (1<<PD6) | (0<<PD5) | (1<<PD4) | (0<<PD3) | (1<<PD2) | (1<<PD1) | (1<<PD0);
            // PortD Daten Register: Enable pullup
            // damit sind bei PortD-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRE Pins teilweise auf Eingabe: kein PE0,PE1,PE3,PE4,PE5,PE7
   DDRE = (1<<DDE6) | (0<<DDE2);
            // PE7 = nicht da                       - in
            // PE6 = D7       CS DAC                - out
            // PE5 = nicht da                       - in
            // PE4 = nicht da                       - in
            // PE3 = nicht da                       - in
            // PE2/HWB = fest auf Masse             - in
            // PE1 = nicht da                       - in
            // PE0 = nicht da                       - in

   // PORTE pullup enable fuer Eingänge: kein PE0,PE1,PE3,PE4,PE5,PE7
   PORTE = (1<<PE6) | (1<<PE2);
            // PortE Daten Register: Enable pullup
            // damit sind bei PortE-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRF Pins teilweise auf Eingabe: kein PF2,PF3
   DDRF = (0<<DDF7) | (0<<DDF6) | (0<<DDF5) | (0<<DDF4) | (0<<DDF1) | (0<<DDF0);
            // PF7 = A0                             - in
            // PF6 = A1                             - in
            // PF5 = A2                             - in
            // PF4 = A3                             - in
            // PF3 = nicht da                       - in
            // PF2 = nicht da                       - in
            // PF1 = A4                             - in
            // PF0 = A5                             - in

   // PORTF pullup enable fuer Eingänge: kein PF2,PF3
   PORTF = (1<<PF7) | (1<<PF6) | (0<<PF5) | (1<<PF4) | (1<<PF1) | (1<<PF0);
            // PortF Daten Register: Enable pullup
            // damit sind bei PortF-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // initialisiere UART lt. Table 19-12. Datenblatt ATmega328P S.203
            // bei 16MHz U2X0=1 38.4k -> 51 Fehler 0.2%
            //                  57.6k    34        0.8%
            //                  115.2k   16        2.1%
            //                  250k      7        0%
            //                  0.5M      3        0%
            //                  1M        1        0%
   //usart_init(51);  // serielle Schnittstelle init mit 38.4kbd, kein FIFO 
   //usart_init(16);  // serielle Schnittstelle init mit 115.2kbd, kein FIFO 
   usart_init(7);     // ser. Schnittstelle init mit 256kbd, kein FIFO 

   // Test USART
   //outc(USART,'A'); // Zeichen zur Bestaetigung ueber UART ruecksenden 
   outc(USART,0xff);  // Zeichen zur Bestaetigung ueber UART ruecksenden 
   outc(USART,'\n');  // neue Zeile ueber UART ruecksenden 
   //outc(USB,'A');   // Zeichen zur Bestaetigung ueber USB ruecksenden - nichts zu sehen
   //outc(USB,'\n');  // neue Zeile ueber USB ruecksenden - nichts zu sehen
   
   
   spi_init(); 
   //usb_init();        // USB init: siehe usb_serial.h 
   
   //adc_init(); 
}

