// Ausgabe-Treiber   Dr. Weisser 24.11.2016  15.10.2019
//***************************************************************************
//  File........: out.c
//  Target(s)...: Atmega328p, Atmega2560, Atmega32u2, Atmega32u4
//  Compiler....: AVR-GCC
//  Description.: routines for output USART/LCD/SRAM/USB
//                outc    -> Zeichen ausgeben
//                outs    -> String aus SRAM ausgeben
//                outs_p  -> String aus Programmspeicher (Flash) ausgeben
//                out_u16bin -> binaere Wortausgabe
//                out_u16 -> Zahlen-Ausgabe
//                out_u32 -> Zahlen-Ausgabe
//                out_u08 -> Zahlen-Ausgabe
//***************************************************************************
/* 24.11.2016 Code aus lcd44780.c und usart.c herausgeloest
    5.12.2016 Ausgabefehler korrigiert bei Wert 0.
              es soll mindestens eine 0 ausgegeben werden.
              dies ist nicht der Fall bei ganzen Zahlen
              -> Funktionen ergaenzt um Term ziffern==1:
              if ((zifferout==1) || (vorkomma==1) || (ziffern==1))
    6.12.2016 neue Funktion out_u08_2 fuer Uhrausgabe mit Vornull
     1.2.2018 Problem mit Ausgabe long mit out_u32:
              10000000 ausgegeben als EEE6EEE.E bei 7,1
     2.2.2018 Teilen mit div Unsinn wenn Teiler 10000000 und in int passen soll
              so wird falsch gerechnet und dann auch angezeigt
              unsigned int div, temp; -> unsigned long div, temp;
              spart Codegroesse 2968 -> 2916 bytes
              bei Ausgabe u08 unsigned int div, temp; -> unsigned char div, temp;
              spart Codegroesse 2916 -> 2890 bytes
              bei Ausgabe u16 bringt 
              unsigned int div, temp; -> unsigned char temp; nicht weniger Code
              -> unsigned int div, temp; bei u16 belassen.
     2.3.2019 Ausgabe um SRAM erweitert 
              Ergaenzungen in outc() und outs(). outs() nutzt outc() 
              beachte dass vor der Ausgabe und nach Ausgabe 
                - SRAM-Adresse, 
                - CS, 
                - SPI-Mode, 
                - SPI-Speed, 
                - spi_on/spi_off 
              sinnvoll zu setzen sind. 
     3.3.2019 neue Funktion out_u16bin() fuer binaer-Ausgabe 
     9.3.2019 Fehler in out_u16bin() behoben 
    27.4.2019 Unterscheidung USART0, USART1 bei atmega328p/2560/32u2 
    24.6.2019 Unterscheidung USART0, USART1 auch bei atmega32u4 
    30.6.2019 Unterscheidung USART0, USART1 automatisch nun 
     1.7.2019 USB-Ausgabe hinzu 
   14.10.2019 Fehler in USB-Ausgabe behoben, Zeichen werden nun gesendet 
   15.10.2019 klare Unterscheidung Stringausgaben SRAM/Flash durch 
              Verwendung outs() fuer SRAM und outs_p() fuer Programmspeicher (Flash) 

*/

#include <stdint.h>       // Integer-Datentypen 8-64bit
#include <avr/io.h>       // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include "main.h"         // CPU-Frequenz, sbi, cbi
#include "delay.h"        // Wartezeiten us, ms
#include "out.h"          // Ausgabe-Defines + Funktions-Prototypen
#include "lcd44780.h"     // fuer Ausgabe auf LCD
#include "spi.h"          // fuer Ausgabe auf SRAM per SPI
#include "usb_serial.h"   // fuer Ausgabe auf USB
#include "usart.h"        // USART-Funktionen fuer Debug-Ausgaben
#include <avr/pgmspace.h> // PSTR


// devices fuer Ausgabe in out.h
//#define LCD   0  // LCD-Funktionsteile ggf. abschalten 
//#define SRAM  0  // SRAM-Ausgabe ggf. abschalten 
//#define USB   0  // USB-Ausgabe 32u4 etc. ggf. abschalten 
//#define USART 0  // USART-Ausgabe ggf. abschalten (wozu?) 

// diese defines in usart.h
/*
#ifndef __AVR_ATmega32U4__   // sollte ueber Makefile dem Compiler bekannt sein 
#define __AVR_ATmega32U4__ 1 // Hardwarewahl: ATmega32U4 
#endif // Hardwarewahl ATmega2560, ATmega328P, ATmega32U2, ATmega32U4..
*/


// schreibt ein Zeichen zur Ausgabe USART/LCD/SRAM/USB
void outc(unsigned char device, unsigned char data)
{
   if (device==USB){ 
#if USB // USB-Ausgabe ggf. ausblenden weil kein USB verbaut etc. 
      // schreibt ein Zeichen zum USB falls USB aktiv/USB-Treiber da 
      usb_serial_putchar(data); // schicke byte nach USB (benoetigt usb_serial.c) 
                                // diese Funktion nutzt ein signed Integer 
                                // wozu dies? reicht kein Character? 
                                // werden immer gleichzeitig 2 bytes auf USB geschickt? 
                                // ist hier casting sinnvoll/noetig? 
      //usb_serial_putchar_nowait(data); // schicke byte nach USB sofort 
                                // ohne zu warten bis Buffer voll ist 
                                // sollte nicht noetig sein wegen
                                // #define TRANSMIT_FLUSH_TIMEOUT   5   /* ms */
                                // alle 5ms wird sptestens das Paket gesendet 
#endif // USB 
   }
   else if (device==USART){ 
#if defined(__AVR_ATmega328P__) || defined (__AVR_ATmega2560__) 
                             // Atmega328/Arduino nano3 oder
                             // Atmega2560 
      // nutze USART0 
      while (!(UCSR0A & (1<<UDRE0)));  // schauen ob Senden ok ist
      UDR0 = data;   // Datenbyte abschicken, wenn erlaubt

#elif defined(__AVR_ATmega32U2__) || defined(__AVR_ATmega32U4__) || defined(__AVR_AT90USB1286__) 
                             // Atmega32U2 oder 
                             // Atmega32U4 oder 
                             // AT90USB1286 
      // nutze USART1 
      while (!(UCSR1A & (1<<UDRE1)));  // schauen ob Senden ok ist
      UDR1 = data;   // Datenbyte abschicken, wenn erlaubt

#endif // Hardwareumschaltung 
   }
   else if (device==LCD){ 
#if LCD // Code fuer LCD-Ausgabe ggf. ausblenden
      // schreibt ein Zeichen in die LCD-Anzeige, RS=1 !
      // deutsche Umlaute ermoeglichen
      // diese Zeichen muessen im Zeichensatz enthalten sein:
      // da die Controller unterschiedlich programmiert sind muss
      // ggf. diese Ersetzung angepasst werden

      switch(data){

      // Umlaute auf LCD ermoeglichen
      case '' : data=0xE1; break;  // Umlenkung auf Zeichen im ROM
      case '' : data=0xEF; break;
      case '' : data=0xF5; break;
      case '' : data=0xE2; break;

      // ggf. Umlenkung auf seltdefinierte Zeichen im CGRAM
      // case '' : data=0x00; break;  // Umlenkung ins CGRAM
      // case '' : data=0x01; break;
      // case '' : data=0x02; break;
      }

      // fuer Ausgabe "0" =0x30 schreiben
      // erst 3 schreiben, dann 0 schreiben: 0011 0000
      // erst obere 4 bit ausgeben:
      // dazu untere 4 bit loeschen, dann verodern mit LCD_RS
      lcd_write(data, led | LCD_RS); // per I2C an HD44780
      delay_us(37);         // 37us warten
#endif // LCD
   }
   else { // SRAM-Ausgabe
#if SRAM // SRAM-Ausgabe ggf. ausblenden 
      // vorher sicherstellen: 
      //   - CS SRAM low -> 
      //   - Adresse im SRAM waehlen
      //   - SPI-Mode 0
      //   - SPI-Speed8
      //   - SPI aktiv
      spi_send_rcv(data); // schreibe byte
      // nach dem Schreiben aller gewuenschten bytes sicherstellen:
      //   - CS high machen
      //   - SPI deaktiv schalten 
#endif // SRAM
   }
}



// schreibt String (Text) aus SRAM zur Ausgabe USART/LCD/SRAM/USB
void outs(unsigned char device, const char *str)
{

   // Ausgabe aus SRAM auf device 
   while (*str) {          // String aus SRAM, get next char, bis /0
      outc(device,*str++); // send char
   } 
} 



// schreibt String (Text) aus Programmspeicher zur Ausgabe USART/LCD/SRAM/USB
void outs_p(unsigned char device, const char *str)
{
   char c;

   // Ausgabe aus Programmspeicher auf device 
   while ((c=pgm_read_byte(str++)) ){  // String aus Programmspeicher, get next char, bis /0
      outc(device,c);      // send char
   } 
}



//****************************************************
// schreibt 16bit-Zahl binaer als 2 byte auf USART/LCD/SRAM/USB
//****************************************************
void out_u16bin(unsigned char device, unsigned int zahl)
{
    unsigned char byte; 

    byte=zahl>>8;      // oberstes byte 8bit nach unten schieben 

    outc(device,byte); // oberstes byte zuerst ausgeben 
    
    byte=zahl&0xFF;    // unteres byte herausmaskieren 
    
    outc(device,byte); // unteres byte ausgeben 

}



//****************************************************
// schreibt positive Zahlen mit/ohne Komma auf USART/LCD/SRAM/USB
// maximal 5 Stellen:
//
// vorkomma=2, nachkomma=1:   580 -> 58.0   feste Stellen vor+nach Komma
//                             58 ->  5.8
//                           5800 -> E0.0   -> Ueberlauf
// vorkomma=3, nachkomma=2: 65432 -> 654.32 3 Stellen vor, 2 Stellen nach Komma
//
// vorkomma=0, nachkomma=1: 65432 -> 6543.2 so viele Stellen wie noetig, 1 Stelle nach Komma
// vorkomma=0, nachkomma=2:  5432 -> 54.32  so viele Stellen wie noetig, 2 Stellen nach Komma
//
// vorkomma=1, nachkomma=0:     1 -> 1      keine Kommaausgabe, genau 1 Stelle
// vorkomma=2, nachkomma=0:    23 -> 23     keine Kommaausgabe, genau 2 Stellen
// vorkomma=3, nachkomma=0:   523 -> 523    keine Kommaausgabe, genau 3 Stellen
// vorkomma=5, nachkomma=0: 52304 -> 52304  keine Kommaausgabe, genau 3 Stellen
//
// vorkomma=0, nachkomma=0:   580 -> 580    keine Kommaausgabe, so viele Stellen wie noetig
//                             58 -> 58
//                           5800 -> 5800
//                          65432 -> 65432
//****************************************************
void out_u16(unsigned char device, unsigned int zahl, unsigned char vorkomma, unsigned char nachkomma)
{
   unsigned char ziffern, zifferout=0;
   unsigned char space;  // ggf. fuehrende Space ausgeben
   unsigned int div, temp;
   //unsigned int div;     // reicht fuer divider[]
   //unsigned char temp;   // reicht als char fuer eine Ziffer, gibt jedoch gleiche Codegroesse
   const unsigned int divider[]={1,10,100,1000,10000};
   unsigned char z_max=5; // maximale Ziffern (65535)

   // zulaessige Bereiche:
   // zahl:      0 .. 65535 (max. 5 Stellen)
   // vorkomma:  0 .. 255   (max. 5 Stellen sinnvoll)
   // nachkomma: 0 .. 255   (max. 5 Stellen sinnvoll)
   // ziffern:   0 .. 255   (max. 5 Stellen sinnvoll)

   // Kombinationen vorkomma, nachkomma auswerten:
   // vor nach
   //  1    1   Komma-Ausgabe definierte Stellenzahl x.x
   //  1    2   Komma-Ausgabe definierte Stellenzahl x.xx
   //  2    3   Komma-Ausgabe definierte Stellenzahl xx.xxx
   //           space=1 (Abstand vorne ausgeben zur Positionierung Display)
   //           ziffern=vorkomma+nachkomma
   //
   //  0    1   Komma-Ausgabe noetige Stellenzahl xx.x
   //  0    2   Komma-Ausgabe noetige Stellenzahl xxx.xx
   //  0    3   Komma-Ausgabe noetige Stellenzahl x.xxx
   //           space=0 (keinen Abstand vorne ausgeben)
   //           ziffern=5
   //           vorkomma=5-nachkomma
   //
   //  1    0   Ganzzahlausgabe 1 Stelle  x
   //  2    0   Ganzzahlausgabe 2 Stellen xx
   //  3    0   Ganzzahlausgabe 3 Stellen xxx
   //  5    0   Ganzzahlausgabe 5 Stellen xxxxx
   //           space=1 (Abstand vorne ausgeben zur Positionierung Display)
   //           ziffern=vorkomma+nachkomma
   //
   //  0    0   Ganzzahlausgabe noetige Stellenzahl xxx
   //           space=0 (keinen Abstand vorne ausgeben)
   //           ziffern=5

   // Schnittstelle:
   // zahl, vorkomma, nachkomma, z_max, device(USART,LCD,SRAM)
   ziffern=vorkomma+nachkomma;

   if (ziffern==0){              // Ganzzahlausgabe mit mindest noetiger Stellenzahl
      ziffern=z_max;             // alle noetigen Stellen darstellen
      space=0;                   // spaces am Anfang unterdruecken
   }
   else if (nachkomma==0){       // Ganzzahlausgabe mit fester Stellenzahl lt. vorkomma
      space=1;                   // spaces am Anfang einbauen fuer feste Stellenzahl
   }
   else if (vorkomma==0){        // Komma-Ausgabe mit mindest noetiger Stellenzahl
      ziffern=z_max;             // alle noetigen Stellen darstellen
      space=0;                   // spaces am Anfang unterdruecken
      vorkomma=z_max-nachkomma;  // korrektes vorkomma berechnen
   }
   else {  // weder vorkomma=0 noch nachkomma=0 -> Komma-Ausgabe mit fester Stellenzahl
      space=1;
   }


   // berechne Teiler div=10 hoch (ziffern-1)
   if (ziffern>z_max)            // verhindere fehlerhaften Array-Zugriff
      ziffern=z_max;
   div=divider[ziffern-1];       // Teiler bestimmen


   for (; ziffern>0; ziffern--){
      temp=zahl/div;             // Wert der hoechsten Ziffer: z.B. 6 bei 6531

      if ((vorkomma==0) && (zifferout==1)){
         outc(device,'.');       // Komma ausgeben, wenn Zahl ausgegeben
      }

      if (temp>=10){             // zu grosse Zahl nicht darstellbar
         outc(device,'E');       // -> E ausgeben fuer overflow-Fehler
                                 // darstellbar waere ggf. noch A,B,C,D,E,F
                                 // falls temp=10,11,12,13,14,15
                                 // und dann anderes Zeichen fuer Overflow
                                 // nur wem wuerde das nuetzen? schafft Verwirrung !
         zifferout=1;            // damit folgende Nullen angezeigt werden
      }
      else if (temp==0){         // fuehrende Nullen unterdruecken
         if ((zifferout==1) || (vorkomma==1) || (ziffern==1)){  // 0 nur dann ausgeben,
                                 // wenn bereits eine Ziffer ausgegeben oder Stelle vor Komma
                                 // oder letzte Ziffer
            outc(device,temp+0x30); // Ziffer 0 als ASCII ausgeben
            zifferout=1;         // Merker, dass eine Ziffer ausgegeben
         }
         else {
            if (space)
               outc(device,' '); // Ziffer 0 unterdruecken -> Leerzeichen stattdessen
         }
      }
      else {
         outc(device,temp+0x30); // Ziffern 1-9 als ASCII ausgeben
         zifferout=1;            // Merker, dass eine Ziffer ausgegeben
      }
      zahl=zahl-temp*div;        // Rest berechnen: z.B. 531
      vorkomma--;
      div=div/10;
   }
}



//****************************************************
// schreibt positive Zahlen mit/ohne Komma auf USART/LCD/SRAM/USB
// maximal 3 Stellen: vgl. out_u16
//****************************************************
void out_u08(unsigned char device, unsigned char zahl, unsigned char vorkomma, unsigned char nachkomma)
{
   unsigned char ziffern, zifferout=0;
   unsigned char space;  // ggf. fuehrende Space ausgeben
   //unsigned int div, temp;
   unsigned char div, temp;  // reicht als char fuer Ziffern und divider[], spart Codegroesse
   const unsigned int divider[]={1,10,100};
   unsigned char z_max=3; // maximale Ziffern (255)

   // zulaessige Bereiche:
   // zahl:      0 .. 255 (max. 3 Stellen)
   // vorkomma:  0 .. 255 (max. 3 Stellen sinnvoll)
   // nachkomma: 0 .. 255 (max. 3 Stellen sinnvoll)
   // ziffern:   0 .. 255 (max. 3 Stellen sinnvoll)

   // Kombinationen vorkomma, nachkomma auswerten:
   // vor nach
   //  1    1   Komma-Ausgabe definierte Stellenzahl x.x
   //  1    2   Komma-Ausgabe definierte Stellenzahl x.xx
   //  2    1   Komma-Ausgabe definierte Stellenzahl xx.x
   //           space=1 (Abstand vorne ausgeben zur Positionierung Display)
   //           ziffern=vorkomma+nachkomma
   //
   //  0    1   Komma-Ausgabe noetige Stellenzahl xx.x
   //  0    2   Komma-Ausgabe noetige Stellenzahl x.xx
   //           space=0 (keinen Abstand vorne ausgeben)
   //           ziffern=3
   //           vorkomma=3-nachkomma
   //
   //  1    0   Ganzzahlausgabe 1 Stelle  x
   //  2    0   Ganzzahlausgabe 2 Stellen xx
   //  3    0   Ganzzahlausgabe 3 Stellen xxx
   //           space=1 (Abstand vorne ausgeben zur Positionierung Display)
   //           ziffern=vorkomma+nachkomma
   //
   //  0    0   Ganzzahlausgabe noetige Stellenzahl xxx
   //           space=0 (keinen Abstand vorne ausgeben)
   //           ziffern=3

   // Schnittstelle:
   // zahl, vorkomma, nachkomma, z_max, device(USART,LCD,SRAM)
   ziffern=vorkomma+nachkomma;

   if (ziffern==0){              // Ganzzahlausgabe mit mindest noetiger Stellenzahl
      ziffern=z_max;             // alle noetigen Stellen darstellen
      space=0;                   // spaces am Anfang unterdruecken
   }
   else if (nachkomma==0){       // Ganzzahlausgabe mit fester Stellenzahl lt. vorkomma
      space=1;                   // spaces am Anfang einbauen fuer feste Stellenzahl
   }
   else if (vorkomma==0){        // Komma-Ausgabe mit mindest noetiger Stellenzahl
      ziffern=z_max;             // alle noetigen Stellen darstellen
      space=0;                   // spaces am Anfang unterdruecken
      vorkomma=z_max-nachkomma;  // korrektes vorkomma berechnen
   }
   else {  // weder vorkomma=0 noch nachkomma=0 -> Komma-Ausgabe mit fester Stellenzahl
      space=1;
   }


   // berechne Teiler div=10 hoch (ziffern-1)
   if (ziffern>z_max)            // verhindere fehlerhaften Array-Zugriff
      ziffern=z_max;
   div=divider[ziffern-1];       // Teiler bestimmen


   for (; ziffern>0; ziffern--){
      temp=zahl/div;             // Wert der hoechsten Ziffer: z.B. 6 bei 6531

      if ((vorkomma==0) && (zifferout==1)){
         outc(device,'.');       // Komma ausgeben, wenn Zahl ausgegeben
      }

      if (temp>=10){             // zu grosse Zahl nicht darstellbar
         outc(device,'E');       // -> E ausgeben fuer overflow-Fehler
                                 // darstellbar waere ggf. noch A,B,C,D,E,F
                                 // falls temp=10,11,12,13,14,15
                                 // und dann anderes Zeichen fuer Overflow
                                 // nur wem wuerde das nuetzen? schafft Verwirrung !
         zifferout=1;            // damit folgende Nullen angezeigt werden
      }
      else if (temp==0){         // fuehrende Nullen unterdruecken
         if ((zifferout==1) || (vorkomma==1) || (ziffern==1)){  // 0 nur dann ausgeben,
                                 // wenn bereits eine Ziffer ausgegeben oder Stelle vor Komma
                                 // oder letzte Ziffer
            outc(device,temp+0x30); // Ziffer 0 als ASCII ausgeben
            zifferout=1;         // Merker, dass eine Ziffer ausgegeben
         }
         else {
            if (space)
               outc(device,' '); // Ziffer 0 unterdruecken -> Leerzeichen stattdessen
         }
      }
      else {
         outc(device,temp+0x30); // Ziffern 1-9 als ASCII ausgeben
         zifferout=1;            // Merker, dass eine Ziffer ausgegeben
      }
      zahl=zahl-temp*div;        // Rest berechnen: z.B. 531
      vorkomma--;
      div=div/10;
   }
}



//****************************************************
// schreibt positive Zahlen ohne Komma auf USART/LCD/SRAM/USB
// genau 2 Stellen mit fuehrender Null
// Ausgabe bis 99, darueber Ex
//****************************************************
void out_u08_2(unsigned char device, unsigned char zahl)
{
   unsigned char high = 0;

   while (zahl >= 10){
      high++;       // Zehnerstelle hochzaehlen
      zahl -= 10;
   }

   // erst high ausgeben
   if (high > 9)    // Ueberlauf da?
      outc(device,'E'); // Ueberlauf-Zeichen ausgeben
   else
      outc(device,high+0x30); // Ziffern 1-9 als ASCII ausgeben

   // dann low ausgeben
   outc(device,zahl+0x30);    // Ziffern 1-9 als ASCII ausgeben

}



//****************************************************
// schreibt positive Zahlen mit/ohne Komma auf USART/LCD/SRAM/USB
// maximal 10 Stellen: vgl. out_u16
//****************************************************
void out_u32(unsigned char device, unsigned long zahl, unsigned char vorkomma, unsigned char nachkomma)
{
   unsigned char ziffern, zifferout=0;
   unsigned char space;      // ggf. fuehrende Space ausgeben
   unsigned long div;        // damit grosse Zahlen wie aus divider[] hineinpassen
   unsigned char temp;       // reicht als char fuer eine Ziffer -> 52 byte weniger Codegroesse
   const unsigned long divider[]={1,10,100,1000,10000,100000,1000000,10000000,100000000,1000000000};
   unsigned char z_max=10;   // maximale Ziffern (4294967295)

   // zulaessige Bereiche:
   // zahl:      0 .. 4294967295 (max. 10 Stellen)
   // vorkomma:  0 .. 255   (max. 10 Stellen sinnvoll)
   // nachkomma: 0 .. 255   (max. 10 Stellen sinnvoll)
   // ziffern:   0 .. 255   (max. 10 Stellen sinnvoll)

   // Kombinationen vorkomma, nachkomma auswerten:
   // vor nach
   //  1    1   Komma-Ausgabe definierte Stellenzahl x.x
   //  1    2   Komma-Ausgabe definierte Stellenzahl x.xx
   //  2    3   Komma-Ausgabe definierte Stellenzahl xx.xxx
   //           space=1 (Abstand vorne ausgeben zur Positionierung Display)
   //           ziffern=vorkomma+nachkomma
   //
   //  0    1   Komma-Ausgabe noetige Stellenzahl xx.x
   //  0    2   Komma-Ausgabe noetige Stellenzahl xxx.xx
   //  0    3   Komma-Ausgabe noetige Stellenzahl x.xxx
   //           space=0 (keinen Abstand vorne ausgeben)
   //           ziffern=10
   //           vorkomma=10-nachkomma
   //
   //  1    0   Ganzzahlausgabe 1 Stelle  x
   //  2    0   Ganzzahlausgabe 2 Stellen xx
   //  3    0   Ganzzahlausgabe 3 Stellen xxx
   //  5    0   Ganzzahlausgabe 5 Stellen xxxxx
   //           space=1 (Abstand vorne ausgeben zur Positionierung Display)
   //           ziffern=vorkomma+nachkomma
   //
   //  0    0   Ganzzahlausgabe noetige Stellenzahl xxx
   //           space=0 (keinen Abstand vorne ausgeben)
   //           ziffern=10

   // Schnittstelle:
   // zahl, vorkomma, nachkomma, z_max, device(USART,LCD,SRAM)
   ziffern=vorkomma+nachkomma;

   if (ziffern==0){              // Ganzzahlausgabe mit mindest noetiger Stellenzahl
      ziffern=z_max;             // alle noetigen Stellen darstellen
      space=0;                   // spaces am Anfang unterdruecken
   }
   else if (nachkomma==0){       // Ganzzahlausgabe mit fester Stellenzahl lt. vorkomma
      space=1;                   // spaces am Anfang einbauen fuer feste Stellenzahl
   }
   else if (vorkomma==0){        // Komma-Ausgabe mit mindest noetiger Stellenzahl
      ziffern=z_max;             // alle noetigen Stellen darstellen
      space=0;                   // spaces am Anfang unterdruecken
      vorkomma=z_max-nachkomma;  // korrektes vorkomma berechnen
   }
   else {  // weder vorkomma=0 noch nachkomma=0 -> Komma-Ausgabe mit fester Stellenzahl
      space=1;
   }


   // berechne Teiler div=10 hoch (ziffern-1)
   if (ziffern>z_max)            // verhindere fehlerhaften Array-Zugriff
      ziffern=z_max;
   div=divider[ziffern-1];       // Teiler bestimmen


   for (; ziffern>0; ziffern--){
      temp=zahl/div;             // Wert der hoechsten Ziffer: z.B. 6 bei 6531

      if ((vorkomma==0) && (zifferout==1)){
         outc(device,'.');       // Komma ausgeben, wenn Zahl ausgegeben
      }

      if (temp>=10){             // zu grosse Zahl nicht darstellbar
         outc(device,'E');       // -> E ausgeben fuer overflow-Fehler
                                 // darstellbar waere ggf. noch A,B,C,D,E,F
                                 // falls temp=10,11,12,13,14,15
                                 // und dann anderes Zeichen fuer Overflow
                                 // nur wem wuerde das nuetzen? schafft Verwirrung !
         zifferout=1;            // damit folgende Nullen angezeigt werden
      }
      else if (temp==0){         // fuehrende Nullen unterdruecken
         if ((zifferout==1) || (vorkomma==1) || (ziffern==1)){  // 0 nur dann ausgeben,
                                 // wenn bereits eine Ziffer ausgegeben oder Stelle vor Komma
                                 // oder letzte Ziffer
            outc(device,temp+0x30); // Ziffer 0 als ASCII ausgeben
            zifferout=1;         // Merker, dass eine Ziffer ausgegeben
         }
         else {
            if (space)
               outc(device,' '); // Ziffer 0 unterdruecken -> Leerzeichen stattdessen
         }
      }
      else {
         outc(device,temp+0x30); // Ziffern 1-9 als ASCII ausgeben
         zifferout=1;            // Merker, dass eine Ziffer ausgegeben
      }
      zahl=zahl-temp*div;        // Rest berechnen: z.B. 531
      vorkomma--;
      div=div/10;
   }
}

