//spi.h

#ifndef SPI_H_
#define SPI_H_

// SPCR-Register:
                                // SPIE SPI-Interrupt-Enable
                                //  SPE SPI-Enable
                                //   DORD Reihenfolge LSB,MSB
                                //    MSTR SPI Master/Slave
                                //      CPOL
                                //       CPHA
                                //        SPR1
                                //         SPR0
                                // 0000 0000
                                //      CPOL=0
                                //       CPHA=0
#define SPI_MODE0         0x00  // 0000 0000
                                //      CPOL=0
                                //       CPHA=1
#define SPI_MODE1         0x04  // 0000 0100
                                //      CPOL=1
                                //       CPHA=0
#define SPI_MODE2         0x08  // 0000 1000
                                //      CPOL=1
                                //       CPHA=1
#define SPI_MODE3         0x0C  // 0000 1100
                                //      CPOL=1
                                //       CPHA=1
#define SPI_MODE_MASK     0x0C  // 0000 1100
                                //        SPR1=0
                                //         SPR0=0
#define SPI_CLOCK_DIV4    0x00  // 0000 0000 -> 4MHz bei 16MHz Takt
                                //        SPR1=0
                                //         SPR0=1
#define SPI_CLOCK_DIV16   0x01  // 0000 0001 -> 1MHz bei 16MHz Takt
                                //        SPR1=1
                                //         SPR0=0
#define SPI_CLOCK_DIV64   0x02  // 0000 0010 -> 250kHz bei 16MHz Takt
                                //        SPR1=1
                                //         SPR0=1
#define SPI_CLOCK_DIV128  0x03  // 0000 0011 -> 125kHz bei 16MHz Takt
                                //        SPR1=1
                                //         SPR0=1
#define SPI_CLOCK_MASK    0x03  // 0000 0011

// SPCR-Register:
                                //         SPI2X
                                // 0000 0000
                                //         SPI2X=1
#define SPI_2XCLOCK       0x01  // 0000 0001 SPI-Takt verdoppelt
                                //         SPI2X=1
#define SPI_2XCLOCK_MASK  0x01  // 0000 0001

// SPI-Takt: SPI2X SPR1 SPR0
        //     1    0    0    fosc/2 = 8 MHz bei 16MHz-Takt
        //     0    0    0    fosc/4 = 4 MHz
        //     1    0    1    fosc/8 = 2 MHz
        //     0    0    1    fosc/16= 1 MHz
        //     1    1    0    fosc/32= 500kHz
        //     1    1    1    fosc/64= 250kHz
        //     0    1    1    fosc/128=125kHz


void spi_init(void);   // SPI Master, SPI 8 MHz bei 16 MHz-Takt 
void spi_speed8(void); // SPI 8 MHz 
void spi_speed4(void); // SPI 4 MHz 
void spi_mode0(void);  // SPI Mode 0
void spi_mode1(void);  // SPI Mode 1
void spi_mode2(void);  // SPI Mode 2
void spi_mode3(void);  // SPI Mode 3
void spi_on(void);     // SPI aktiv schalten
void spi_off(void);    // SPI deaktiv schalten
unsigned char spi_send_rcv(unsigned char out); // per SPI Byte senden und Byte empfangen
//void spi_send_byte(unsigned char byte); // per SPI byte senden
//unsigned char spi_receive_byte(void)    // per SPI byte empfangen

#endif
