//***************************************************************************
//  File........: timer.c
//  Target(s)...: ATmega328p
//  Compiler....: AVR-GCC
//  Description.: AVR Interrupts
//                Interrupt-Tabelle in io.h -> iom328p.h
//                ISR-Tabelle auch erstellen mit
// $ echo | avr-gcc -mmcu=atmega328 -g3 -xc - -include avr/io.h -E -dM | grep _vect\b | sort
//***************************************************************************

#include <avr/io.h>
#include "timer.h"
#include "usart.h"
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

// Global variables
extern
volatile unsigned int Timer;  // timer fuer Wartezeiten (10ms increment)

/*---------------------------------------------------------*/
/* 10ms-Timer fuer Zeitmessung, Timeouts                   */
/*     Timer/Counter0 OutputCompare Match A                */
/*     Timer0-Ueberlauf alle 10ms -> loest Interrupt aus   */
/*---------------------------------------------------------*/
void Timer0_init(void)
{

   // Timer0 bis OCR0A = F_CPU/1024/100-1
   OCR0A = 155;   // laeuft von 0-155 hoch, dann Interrupt
                  // Compare-Match loescht Timer0 automatisch
   //TCCR0A |= (1<<WGM01);  // set bit WGM01 S.106,108
   TCCR0A = (1<<WGM01);  // set bit WGM01 S.106,108
   //TCCR0B |= (1<<CS02)|(1<<CS00); // Prescaler =1024 (geht auch 0b101)
   TCCR0B = (1<<CS02)|(1<<CS00); // Prescaler =1024 (geht auch 0b101)
   //TIMSK0 |= (1<<OCIE0A); // Timer Interrupt Mask Register
   TIMSK0 = (1<<OCIE0A); // Timer Interrupt Mask Register

   // nicht vergessen Interrupts aktiv zu schalten mit sei()

}


/*---------------------------------------------------------*/
/* 10ms timer interrupt generated by OC0A                  */
/*     dazu Timer0 init mit Teiler durch 1024              */
/*     Timer/Counter0 OutputCompare Match A                */
/*     -> Timer0-Compare-Interrupt bei 155 alle 10ms       */
/*---------------------------------------------------------*/
ISR(TIMER0_COMPA_vect)
{
   Timer++;                // Performance counter
   //mmc_disk_timerproc(); // timer for low level disk I/O module
}
